#!/bin/bash
#
# Solves a triangle and finds some notable values
#
# (C) 2006 - Diego Bravo Estrada
#

: ${DIALOG=dialog}

# Scale for bc calculations
bscale=16

# If we had to rotate the triangle in order to match a solution,
# unrotate at end:
rot_state=0

TMPERR=/tmp/triangle.$$.tmp

function check_tmperr()
{
if [ "`ls -l $TMPERR | awk '{print $5}'`" = 0 ]; then
	echo 0
else
	echo 1
fi
}

function rotate()
{
tmp_s="$t_s_a"
tmp_a="$t_a_a"
t_s_a="$t_s_b"
t_a_a="$t_a_b"
t_s_b="$t_s_c"
t_a_b="$t_a_c"
t_s_c="$tmp_s"
t_a_c="$tmp_a"

if [ $rot_state = 0 ]; then rot_state=1 ; return ; fi
if [ $rot_state = 1 ]; then rot_state=2 ; return ; fi
if [ $rot_state = 2 ]; then rot_state=0 ; return ; fi
}

function find_altures()
{
alt_c=`echo "scale=${bscale};2*$area/$t_s_c"| bc -l`
alt_a=`echo "scale=${bscale};2*$area/$t_s_a"| bc -l`
alt_b=`echo "scale=${bscale};2*$area/$t_s_b"| bc -l`
}

function find_bisectors()
{
bis_c=`echo "scale=${bscale};sqrt($t_s_a*$t_s_b-$t_s_c*$t_s_c*$t_s_a*$t_s_b/(($t_s_a+$t_s_b)*($t_s_a+$t_s_b)))"| bc -l`
bis_a=`echo "scale=${bscale};sqrt($t_s_b*$t_s_c-$t_s_a*$t_s_a*$t_s_b*$t_s_c/(($t_s_b+$t_s_c)*($t_s_b+$t_s_c)))"| bc -l`
bis_b=`echo "scale=${bscale};sqrt($t_s_c*$t_s_a-$t_s_b*$t_s_b*$t_s_c*$t_s_a/(($t_s_c+$t_s_a)*($t_s_c+$t_s_a)))"| bc -l`
}

function find_meds()
{
med_c=`echo "scale=${bscale};sqrt(($t_s_a*$t_s_a+$t_s_b*$t_s_b-$t_s_c*$t_s_c/2)/2)"| bc -l`
med_a=`echo "scale=${bscale};sqrt(($t_s_b*$t_s_b+$t_s_c*$t_s_c-$t_s_a*$t_s_a/2)/2)"| bc -l`
med_b=`echo "scale=${bscale};sqrt(($t_s_c*$t_s_c+$t_s_a*$t_s_a-$t_s_b*$t_s_b/2)/2)"| bc -l`
}

function find_area()
{
area=`echo "scale=${bscale};p=($t_s_a+$t_s_b+$t_s_c)/2;sqrt(p*(p-$t_s_a)*(p-$t_s_b)*(p-$t_s_c))" | bc -l`
}

function find_inradius()
{
inradius=`echo "scale=${bscale};p=($t_s_a+$t_s_b+$t_s_c)/2;$area/p" | bc -l`
}

function find_circunradius()
{
circunradius=`echo "scale=${bscale};($t_s_a*$t_s_b*$t_s_c)/(4*$area)" | bc -l`
}

function complete_angles()
{
if [ "$t_a_b" != "" -a "$t_a_c" != "" -a "$t_a_a" = "" ]; then
	anglesum $t_a_b $t_a_c
	t_a_a=$par
fi
if [ "$t_a_b" != "" -a "$t_a_c" = "" -a "$t_a_a" != "" ]; then
	anglesum $t_a_b $t_a_a
	t_a_c=$par
fi
if [ "$t_a_b" = "" -a "$t_a_c" != "" -a "$t_a_a" != "" ]; then
	anglesum $t_a_c $t_a_a
	t_a_b=$par
fi
}

function find_side_with_sin()
{
ax="$1"
Ax="$2"
Cx="$3"
par=`echo "scale=${bscale};pi=a(1)*4;$ax*s($Cx*pi/180)/s($Ax*pi/180)" | bc -l`
return 0
}

function find_side_with_cos()
{
ax="$1"
bx="$2"
Cx="$3"
par=`echo "scale=${bscale};pi=a(1)*4;sqrt($ax*$ax+$bx*$bx-2*$ax*$bx*c(pi*$Cx/180))" | bc -l 2>$TMPERR`
RV="`check_tmperr`"
if [ "$RV" = 1 ]; then
	"$DIALOG" --msgbox "Absurd values for triangle" 5 35
	return 2
else
	return 0
fi
return 0
}

function find_angle_sin()
{
ax="$1"
bx="$2"
Ax="$3"
Bx=`echo "pi=a(1)*4;ang=$Ax*pi/180;sb=$bx*s(ang)/$ax;cb=sqrt(1-sb*sb);ratn=a(sb/cb)*180/pi;scale=${bscale};if(ratn<0)(180+ratn);ratn" | bc -l`
# if BX < AX there is no ambiguity
AMB=`echo "$Bx < $Ax" | bc -l`
if [ "$AMB" = 1 ]; then
	par=$Bx
	return 0
fi
# ask the user which case to use
$DIALOG --title "Ambiguous Case" --clear \
        --yes-label "Acute" --no-label "Obtuse" --yesno \
		 "Sorry, there is an ambiguity. Do you want to make the \
                 triangle acute or obtuse? " 6 61

if [ $? = 0 ]; then
	par=$Bx
	return 0
fi
par=`echo "d=180-$Bx;scale=${bscale};d" | bc -l`
return 0
}

# Returns third angle
function anglesum()
{
ax="$1"
bx="$2"
par=`echo "s=180-$ax-$bx;scale=${bscale};s" | bc -l`
return 0
}

# Returns central angule
function cosines()
{
ax="$1"
bx="$2"
cx="$3"
par=`echo "pi=a(1)*4;c=($ax*$ax+$cx*$cx-$bx*$bx)/(2*$ax*$cx);if(c*c<0.000001)90 else {s=sqrt(1-c*c);ratn=a(s/c)*180/pi;scale=${bscale};if(ratn<0)(180+ratn) else ratn}" | bc -l 2>$TMPERR`
RV="`check_tmperr`"
if [ "$RV" = 1 ]; then
	"$DIALOG" --msgbox "Absurd values for triangle" 5 35
	return 2
else
	return 0
fi
}

function solve()
{
# Known three sides, find angles
if [ "$t_s_a" != "" -a "$t_s_b" != "" -a "$t_s_c" != "" ]; then
	if ! cosines $t_s_a $t_s_b $t_s_c ; then return 2 ; fi
	t_a_b=$par
	if ! cosines $t_s_b $t_s_c $t_s_a ; then return 2 ; fi
	t_a_c=$par
	if ! cosines $t_s_c $t_s_a $t_s_b ; then return 2 ; fi
	t_a_a=$par
	return 0
fi
# Known one side and the angles, find the rest
if [ "$t_s_a" != "" -a "$t_a_b" != "" -a "$t_a_c" != "" -a "$t_a_a" != "" ]; then
	if ! find_side_with_sin $t_s_a $t_a_a $t_a_b ; then return 2 ; fi
	t_s_b=$par
	if ! find_side_with_sin $t_s_a $t_a_a $t_a_c ; then return 2 ; fi
	t_s_c=$par
	return 0
fi
# Known two sides and central angle, find the rest
if [ "$t_s_a" != "" -a "$t_s_b" != "" -a "$t_a_c" != "" ]; then
	if ! find_side_with_cos $t_s_a $t_s_b $t_a_c ; then return 2 ; fi
	t_s_c=$par
	if ! cosines $t_s_a $t_s_b $t_s_c ; then return 2 ; fi
	t_a_b=$par
	if ! cosines $t_s_c $t_s_a $t_s_b ; then return 2 ; fi
	t_a_a=$par
	return 0
fi
return 1
}

function solve_sad()
{
# Known two sides, non central angle, ambiguous case
if [ "$t_s_a" != "" -a "$t_s_b" != "" -a "$t_a_a" != "" ]; then
	if ! find_angle_sin $t_s_a $t_s_b $t_a_a ; then return 2 ; fi
	t_a_b=$par
	if ! anglesum $t_a_b $t_a_a ; then return 2 ; fi
	t_a_c=$par
	if ! find_side_with_cos "$t_s_a" "$t_s_b" "$t_a_c" ; then return 2 ; fi
	t_s_c=$par
	return 0
fi
# The reflection
if [ "$t_s_a" != "" -a "$t_s_b" != "" -a "$t_a_b" != "" ]; then
	if ! find_angle_sin $t_s_b $t_s_a $t_a_b ; then return 2 ; fi
	t_a_a=$par
	if ! anglesum $t_a_a $t_a_b ; then return 2 ; fi
	t_a_c=$par
	if ! find_side_with_cos "$t_s_b" "$t_s_a" "$t_a_c" ; then return 2 ; fi
	t_s_c=$par
	return 0
fi
return 1
}

# Check a positive number was typed
function bad_formed()
{
if [ "$1" = "" ]; then return 0 ; fi
if ! echo "$1" | bc -l >/dev/null 2>&1
then return 1 ; fi
r=`echo "$1+0>0" | bc -l`
if [ "$r" = 1 ]; then return 0 ; fi
return 1
}

# 10 REM * PROGRAM STARTS HERE
if ! command $DIALOG &> /dev/null
then
	echo "Sorry, need $DIALOG external command"
	exit 1
fi
backtitle="Solve a triangle and get its attributes"
returncode=0
while test $returncode != 1 && test $returncode != 250
do
exec 3>&1
value=`$DIALOG --ok-label "Submit" \
	  --backtitle "$backtitle" \
	  --form " Sides and Angles of the triange:\n\
\n\
     Sides              Angles(deg)" 12 41 0 \
	"a:" 1 2 "$t_s_a" 1 5  10 0 \
	"b:" 2 2 "$t_s_b"  2 5  10 0 \
	"c:" 3 2 "$t_s_c"  3 5  10 0 \
	"A:" 1 20 "$t_a_a" 1 24  10 0 \
	"B:" 2 20 "$t_a_b" 2 24  10 0 \
	"C:" 3 20 "$t_a_c" 3 24  10 0 \
2>&1 1>&3`
returncode=$?
exec 3>&-

if test $returncode -eq 1
then exit ; fi
t_s_a=`echo "$value" | sed -n '1p'`
t_s_b=`echo "$value" | sed -n '2p'`
t_s_c=`echo "$value" | sed -n '3p'`
t_a_a=`echo "$value" | sed -n '4p'`
t_a_b=`echo "$value" | sed -n '5p'`
t_a_c=`echo "$value" | sed -n '6p'`

# Check blank or positive values
bad_formed "$t_s_a" || { "$DIALOG" --msgbox "Error on side a" 5 23 ; continue ;}
bad_formed "$t_s_b" || { "$DIALOG" --msgbox "Error on side b" 5 23 ; continue ;}
bad_formed "$t_s_c" || { "$DIALOG" --msgbox "Error on side c" 5 23 ; continue ;}
bad_formed "$t_a_a" || { "$DIALOG" --msgbox "Error on angle A" 5 24 ; continue ;}
bad_formed "$t_a_b" || { "$DIALOG" --msgbox "Error on angle B" 5 24 ; continue ;}
bad_formed "$t_a_c" || { "$DIALOG" --msgbox "Error on angle C" 5 24 ; continue ;}

# If possible, find angles by suplement
complete_angles

# Solve the triangle: first, the non ambiguous cases: use rotations
solve "$t_s_a" "$t_s_b" "$t_s_c" "$t_a_a" "$t_a_b" "$t_a_c"
r=$?
if [ $r = 2 ]; then continue ; fi
if [ $r != 0 ]; then
rotate
solve "$t_s_b" "$t_s_c" "$t_s_a" "$t_a_b" "$t_a_c" "$t_a_a"
r=$?
if [ $r = 2 ]; then continue ; fi
if [ $r != 0 ]; then
rotate
solve "$t_s_c" "$t_s_a" "$t_s_b" "$t_a_c" "$t_a_a" "$t_a_b"
r=$?
if [ $r = 2 ]; then continue ; fi
if [ $r != 0 ]; then
###### start ambiguous case #############
solve_sad "$t_s_a" "$t_s_b" "$t_s_c" "$t_a_a" "$t_a_b" "$t_a_c"
r=$?
if [ $r = 2 ]; then continue ; fi
if [ $r != 0 ]; then
rotate
solve_sad "$t_s_b" "$t_s_c" "$t_s_a" "$t_a_b" "$t_a_c" "$t_a_a"
r=$?
if [ $r = 2 ]; then continue ; fi
if [ $r != 0 ]; then
rotate
solve_sad "$t_s_c" "$t_s_a" "$t_s_b" "$t_a_c" "$t_a_a" "$t_a_b"
r=$?
if [ $r = 2 ]; then continue ; fi
if [ $r != 0 ]; then
	"$DIALOG" --msgbox "Not enough information to solve the triangle" 5 50
	continue
fi
fi
fi
######  end of ambiguous case ############
fi
fi
fi

if [ $rot_state = 2 ]; then
	rotate
	rot_state=0
fi
if [ $rot_state = 1 ]; then
	rotate
	rotate
	rot_state=0
fi
sides=`printf  'SIDES:      a: %-9g    b: %-9g    c: %-9g' "$t_s_a" "$t_s_b" "$t_s_c"`
angles=`printf 'ANGLES:     A: %-9g    B: %-9g    C: %-9g' "$t_a_a" "$t_a_b" "$t_a_c"`
find_area
find_inradius
find_circunradius
find_bisectors
find_meds
find_altures
p_area=`printf  'AREA:          %-9g INRa: %-9g CNRa: %-9g' $area $inradius $circunradius`
p_bis=`printf   'BISECTORS: ba: %-9g   bb: %-9g   bc: %-9g' $bis_a $bis_b $bis_c`
p_med=`printf   'MEDIANS:   ma: %-9g   mb: %-9g   mc: %-9g' $med_a $med_b $med_c`
p_halt=`printf  'ALTURES:   ha: %-9g   hb: %-9g   hc: %-9g' $alt_a $alt_b $alt_c`
"$DIALOG" \
--clear \
--backtitle "$backtitle" --no-collapse --cr-wrap \
--msgbox "Solved Triangle:\n\
\n\
$sides\n\
$angles\n\
\n\
$p_area\n\
$p_bis\n\
$p_med\n\
$p_halt\n\
" 14 70
done
